
# Description

"""
Provides GUI elements for AMT."""


# System imports

import sys
import time
import os
import os.path
import threading
import Queue

import distutils.dir_util
import distutils.file_util

from Tkinter import *
import tkFileDialog
import tkMessageBox
         
# Program imports

import amtconf
import amt
import amthelper

root = Tk() #setup a top level root frame in tkinter

def go_button():

    print("Go_button pushed")
    amt.move_files()


def get_the_to_dir():
        amtconf.to_dir=tkFileDialog.askdirectory(title='Select "Automatically Add to iTunes" folder',
                                                 mustexist=True, initialdir=amtconf.to_dir)

def get_the_from_dir():
        amtconf.from_dir=tkFileDialog.askdirectory(title='Select folder to take files from',
                                                   mustexist=True, initialdir=amtconf.from_dir)

def make_configuration_box(): 

    configbox = Toplevel(root) # new frame
    configbox.geometry('450x250+400+400')
    configbox.lift() # bring to front
    configbox.grab_set() # take all events


    Label(configbox,text="").pack()
    Label(configbox,text="You can adjust the program settings here if you wish.").pack()
    Label(configbox,text="You do not need to adjust these values for normal use.").pack()
    Label(configbox,text="").pack()

    # radio buttons for
    # housekeeping gap delay; intersong gap delay; itunes sort order; move or copy;  

    sort_order_radio = StringVar() 
    sort_order_radio.set(amtconf.sort_order)

    rf1 = Frame(configbox, height=32, width=20)
    rf1.pack()

    Label(rf1,text="iTunes sort order?").pack(side=LEFT)
    Radiobutton(rf1, text='Newest At Top', variable=sort_order_radio, value='NewestAtTop').pack(side=LEFT)
    Radiobutton(rf1, text='Newest At Bottom', variable=sort_order_radio, value='NewestAtBottom').pack(side=LEFT)

    move_or_copy_radio = StringVar()
    move_or_copy_radio.set(amtconf.move_or_copy)

    rf2 = Frame(configbox, height=32, width=20)
    rf2.pack()

    Label(rf2,text="Move or Copy files?").pack(side=LEFT)
    Radiobutton(rf2, text='Move', variable=move_or_copy_radio, value='Move').pack(side=LEFT)
    Radiobutton(rf2, text='Copy', variable=move_or_copy_radio, value='Copy').pack(side=LEFT)

    housekeeping_gap_radio = IntVar()
    housekeeping_gap_radio.set(amtconf.housekeeping_gap)

    rf3 = Frame(configbox, height=32, width=20)
    rf3.pack()

    #put in a frame
    Label(rf3,text="Pause every 200 songs (seconds):").pack(side=LEFT)
    Radiobutton(rf3, text="0", variable=housekeeping_gap_radio, value=0).pack(side=LEFT)
    Radiobutton(rf3, text="10", variable=housekeeping_gap_radio, value=10).pack(side=LEFT)
    Radiobutton(rf3, text="20", variable=housekeeping_gap_radio, value=20).pack(side=LEFT)
    Radiobutton(rf3, text="40", variable=housekeeping_gap_radio, value=40).pack(side=LEFT)
    Radiobutton(rf3, text="80", variable=housekeeping_gap_radio, value=80).pack(side=LEFT)

    song_gap_radio = IntVar()
    song_gap_radio.set(amtconf.song_gap)

    rf4 = Frame(configbox, height=32, width=20)
    rf4.pack()

    #put in a frame
    Label(rf4,text="Pause between songs (seconds):").pack(side=LEFT)
    Radiobutton(rf4, text="0", variable=song_gap_radio, value=0).pack(side=LEFT)
    Radiobutton(rf4, text="1", variable=song_gap_radio, value=1).pack(side=LEFT)
    Radiobutton(rf4, text="2", variable=song_gap_radio, value=2).pack(side=LEFT)
    Radiobutton(rf4, text="4", variable=song_gap_radio, value=4).pack(side=LEFT)
    Radiobutton(rf4, text="8", variable=song_gap_radio, value=8).pack(side=LEFT)

    Label(configbox, text=" ").pack()

    rf0 = Frame(configbox, height=32, width=20)
    rf0.pack()

    def apply_config_changes():
        amtconf.sort_order = sort_order_radio.get()
        amtconf.move_or_copy = move_or_copy_radio.get()
        amtconf.song_gap = song_gap_radio.get()
        amtconf.housekeeping_gap = housekeeping_gap_radio.get()
        configbox.destroy()

    Button(rf0,text='Cancel', command=configbox.destroy).pack(side=LEFT)
    Label(rf0,text='  ').pack(side=LEFT)
    Button(rf0,text='OK', command=apply_config_changes).pack(side=LEFT)

    Label(configbox, text=" ").pack()





def make_progress_box():
    quitbox = Toplevel(root) #new frame  #added amtgui
    quitbox.geometry('350x200+450+425')
    quitbox.lift()
    quitbox.grab_set() #prevent root window from accepting events
    quitboxtext = StringVar(quitbox)
    filecounter = StringVar(quitbox)
    quitboxtext.set("Moving files")
    filecounter.set("Items moved: 0")

    Label(quitbox, text=" ").pack(expand=True)
    Label(quitbox, textvariable=filecounter).pack()
    Label(quitbox, textvariable=quitboxtext).pack()  #make this neater. 
    Label(quitbox, text=" ").pack()
    Button(quitbox, text="Cancel", command=quitbox.destroy).pack()
    Label(quitbox, text=" ").pack(expand=True)
    quitbox.resizable(FALSE,FALSE)

       # assume tkinter not threadsafe. therefore pass events through queue
    def update_loop():
        stopqpickup=False
        while amtconf.q.qsize():
            try:
                msg=amtconf.q.get(0)
                if msg[0] == "DONE":
                    quitbox.destroy()
                    stopqpickup=True
                if msg[0] == "UPDATE":  #assuming only 1 thread putting things in Q
                    quitboxtext.set(msg[1])
                    filecounter.set(msg[2])
            except Queue.Empty:    #shouldn't be needed if only 1 producer thread for Q
                pass
        if not stopqpickup: quitbox.after(100, update_loop) # reschedule this in GUI thread in 100ms
    
    quitbox.after(100,update_loop) # initiate update_loop scheduling
    update_loop() #is this needed
    return quitbox

#amt gui should expose root

def about():
        tkMessageBox.showinfo('Add My Tunes by Graeme Bell',
        "Add My Tunes by Graeme Bell - v"+str(amtconf.current_version)+"\n\n"+
        "This program slowly moves folders so that albums aren't mixed "+
        "up together in iTunes if you view by Date Added. It also "+
        "adds each track in the 'correct order' when viewed by Date Added. "+
        "Files must be moved slowly, please be patient.\n\n"+
        "The program can handle folders inside folders, etc., but please "+
        "do NOT store this program inside the folder you're moving.\n\n"+
        "You can adjust the pauses which take place between each song and also "+
        "every 200 songs. The default values (20 / 1) suit most systems.\n\n"+
        "You can also edit amtconf.py to enable no-GUI automatic mode, or change "+
        "the default starting directories. You can use a program like 'Hazel' to "+
        "automatically run iTunes & Add My Tunes whenever new music is downloaded.\n\n\n"+
        "http://graemebell.net/software/tools/ "+
        "has further information and updates.\n\n",
        parent=root
        )



# GUI starts here - stick in a function or leave script-style?

def new_version_message_box():
    tkMessageBox.showinfo("Please update!", 
        "A major new version has been released.\n\n"+
        "Please visit http://graemebell.net/software/tools/add-my-tunes "+
        "to get the latest version.\n\n"+
        "You can disable this message in amtconf.py", parent=root)

def done_box():
    tkMessageBox.showinfo("Done!","File transfer complete", parent=root)

def warning_box(title, text):
    tkMessageBox.showinfo(title,text,parent=root)



def gui_start():

    root.title('Add My Tunes')
    root.withdraw()

    Frame(root, height=32, width=20).pack()

    Button(root,text='Select folder of MP3 albums', command=get_the_from_dir).pack()
    Label(root,text="").pack()
    Button(root,text='Select "Automatically Add to iTunes" folder', 
        command=get_the_to_dir).pack()
                                
    #Add a gap
    Frame(root, height=32, width=20).pack()
                                
    Button(root,text='Go!', command=go_button).pack()
                                
    #Add a gap
    Frame(root, height=32, width=20).pack()
                        
    # final buttons

    rf0 = Frame(root, height=32, width=20)

    Button(rf0,text='Options', command=make_configuration_box).pack(side=LEFT)
    Label(rf0,text='  ').pack(side=LEFT)
    Button(rf0,text='About', command=about).pack(side=LEFT)
    Label(rf0,text='  ').pack(side=LEFT)
    Button(rf0,text='Quit', command=sys.exit).pack(side=LEFT)

    rf0.pack()

    Frame(root, height=32, width=20).pack()


    root.geometry('450x250+400+400') # move window to middle of screen
    root.lift() # lift window to top
    root.resizable(FALSE,FALSE)


    # Fix for MacOS GUI placement in the background / support for MSWindows
    if os.name == 'posix':
        root.wm_attributes("-topmost",1)

    root.deiconify()

    if os.name == 'posix':
        root.wm_attributes("-topmost",0)

    amthelper.check_version()
    mainloop() # start tkinter running

